<?php
/*
Plugin Name: EasyGeo
Plugin URI: http://theoephraim.com/projects/easygeo/
Description: Provides very simple geocoding features for Wordpress, allowing each post to be tagged with a location (description, latitude, longitude). Additionally has a feature to track of a single location not assocaited with posts (useful to create a "Where Am I?" feature on your blog)
Version: 1.0
Author: Theo Ephraim
Author URI: http://theoephraim.com
Minimum WordPress Version Required: 2.7.0
Tested up to: 3.0.0
*/


/* ==================================================================== */
/* = Hooks, Filters, Globals etc.                                     = */
/* ==================================================================== */

global $easygeo_options;
$easygeo_options = get_option("easygeo_options");

register_activation_hook(__FILE__, array("EasyGeo", "registerPlugin"));
add_action("admin_menu", array("EasyGeo", "hookAdminMenu"));
add_action("admin_footer", array("EasyGeo", "hookAdminFooter"));
add_action("save_post", array("EasyGeo", "hookSavePost"));
add_action("wp_head", array("EasyGeo", "hookWPHeader"));
add_action('wp_dashboard_setup', array("EasyGeo", "hookDashboardSetup"));
add_action("wp_ajax_easygeo_update_current", array("EasyGeo", "hookAjaxUpdateCurrent"));
if ($easygeo_options["GEOTAG_FEEDS"] == "true") {
	add_action("rss2_ns", array("EasyGeo", "hookFeedNamespace"));
	add_action("atom_ns", array("EasyGeo", "hookFeedNamespace"));
	add_action("rdf_ns", array("EasyGeo", "hookFeedNamespace"));
	add_action("rss_item", array("EasyGeo", "hookFeedItem"));
	add_action("rss2_item", array("EasyGeo", "hookFeedItem"));
	add_action("atom_entry", array("EasyGeo", "hookFeedItem"));
	add_action("rdf_item", array("EasyGeo", "hookFeedItem"));
}

class EasyGeo {	
	function registerPlugin() {
		$options = array(
			"gmap_api_key" => "",
			"track_current" => false,
			"current_lat" => "",
			"current_lon" => "",
			"current_loc" => ""
		);
		add_option("easygeo_options", $options);
	}
	
	function hookAdminMenu() {
		add_meta_box("easygeo", "EasyGeo", array("EasyGeo", "displayEditPostForm"), "post", "normal", "low");
		add_meta_box("easygeo", "EasyGeo", array("EasyGeo", "displayEditPostForm"), "page", "normal", "low");
		add_options_page("EasyGeo Configuration", "EasyGeo", 8, __FILE__, array("EasyGeo", "displayOptions"));
	}
	
	function hookAdminFooter() {
		global $easygeo_options;
		if (empty($easygeo_options["gmap_api_key"])) {return;}		//die if API Key isnt set
		
		//determine if were editing a page, or on the dashboard, or neither... Why isnt this built into wordpress???
		$curpage = strrchr($_SERVER['SCRIPT_NAME'],'/');
		$editing = ( $curpage == '/page-new.php' || $curpage == '/post-new.php' || (($curpage == '/post.php' || $curpage == '/page.php') && $_GET['action']=='edit' && current_user_can('edit_post', (int) $_GET['post'])) );
		$dashboard = ($curpage == '/index.php');
				
		if ($editing){
			list($loc, $lat, $lon) = EasyGeo::getCoordinates();
		} else if ($dashboard && $easygeo_options['track_current']){
			list($loc, $lat, $lon) = EasyGeo::getCurrentCoordinates();
		} else {
			//do nothing if not showing the interface
			return;
		}
		
		//make the map center on "current location"?
		echo "
			<script type='text/javascript' src='http://maps.google.com/maps/api/js?sensor=false'></script>
			<script type='text/javascript'>
			  var easygeo_map, easygeo_geocoder, easygeo_lastmarker;
			  function easygeoInitialize() {
			    var myOptions = {
			      zoom: 8,
			      mapTypeId: google.maps.MapTypeId.ROADMAP
			    };
			    easygeo_map = new google.maps.Map(document.getElementById('easygeo_map'), myOptions);
			  	easygeo_geocoder = new google.maps.Geocoder();";
		if ($lat && $lon) echo "
				easygeo_lastmarker = new google.maps.Marker({
            		map: easygeo_map, 
    				position: new google.maps.LatLng('$lat', '$lon'),
    				alt: 'You are here!'	
    			});
			  	easygeo_map.setCenter(easygeo_lastmarker.getPosition());";
		else echo "
				easygeo_map.setCenter(new google.maps.LatLng(40.69847032728747, -73.9514422416687));	//NYC";
		echo "
				jQuery('#easygeo_loc').keypress(function(e){ 
					if(e.keyCode == 13){
						jQuery('#easygeo_search_button').click();
						return false;
					}
				});
			  }
			  function easygeoGeocode(){
			  	var location = document.getElementById('easygeo_loc').value;
			  	if (easygeo_geocoder) {
     				easygeo_geocoder.geocode( { 'address': location}, function(results, status) {
        				//Right now just takes first result
        				if (status == google.maps.GeocoderStatus.OK) {
          					if (easygeo_lastmarker) easygeo_lastmarker.setMap(null);	//removes from the map
          					easygeo_map.setCenter(results[0].geometry.location);
          					easygeo_lastmarker = new google.maps.Marker({
            					map: easygeo_map, 
		        				position: results[0].geometry.location,
		        				alt: 'You are here!'	
		        			});
							//hidden input fields
							document.getElementById('easygeo_update').value='true';
		        			document.getElementById('easygeo_lat').value=results[0].geometry.location.lat();
			  				document.getElementById('easygeo_lon').value=results[0].geometry.location.lng();
							//visible but disabled input fields. They dont actually get submitted but help with UI
							document.getElementById('easygeo_lat_dis').value=results[0].geometry.location.lat();
			  				document.getElementById('easygeo_lon_dis').value=results[0].geometry.location.lng();";
			if ($dashboard) echo "
							//Sends an AJAX request to update the current location
							var data = {
								action     : 'easygeo_update_current',
								loc        : location,
								lat        : results[0].geometry.location.lat(),
								lon        : results[0].geometry.location.lng()	
							};

							// since 2.8 ajaxurl is always defined in the admin header and points to admin-ajax.php
							jQuery.post(ajaxurl, data, function(response) {
								//alert('Got this from the server: ' + response);
								jQuery('#easygeo_geocode_status').text('Saved').show().fadeOut(2000);
							});";
			echo "
   			     		} else {
          					alert('Geocode was not successful for the following reason: ' + status);
          				}
          			});
          		} else {
          			alert('There was a problem initializing the Google Geocoder. Please try refreshing');
          		}
          		return false;
          	  }
			  function easygeoClear(){
			    document.getElementById('easygeo_loc').value='';
			  	document.getElementById('easygeo_lat').value='';
			  	document.getElementById('easygeo_lon').value='';
			  	document.getElementById('easygeo_lat_dis').value='';
			  	document.getElementById('easygeo_lon_dis').value='';
				document.getElementById('easygeo_update').value='true';
			  	if (document.getElementById('easygeo_updatecurrent')) document.getElementById('easygeo_updatecurrent').checked = false;
			  	if (easygeo_lastmarker) easygeo_lastmarker.setMap(null);	//removes from the map
			  }
		  
		  
			  jQuery(document).ready(function(){ easygeoInitialize(); });
			</script>";
	}

	function hookSavePost($post_id) {
		if ( $_POST["easygeo_update"]=='true' && isset($_POST["easygeo_loc"]) && isset($_POST["easygeo_lat"]) && isset($_POST["easygeo_lon"]) ) {
			EasyGeo::putCoordinates($_POST["easygeo_loc"], $_POST["easygeo_lat"], $_POST["easygeo_lon"], $post_id);
			if ( isset($_POST["easygeo_updatecurrent"]) ) {
				EasyGeo::putCurrentCoordinates($_POST["easygeo_loc"], $_POST["easygeo_lat"], $_POST["easygeo_lon"]);
			}
		}
		
	}
	
	/**
	 * Create HTML to display
	 */
	
	function displayEditPostForm($post = null, $args = false, $useCurrent = false) {
		// $useCurrent is whether or not to use the Current location instead of the post location. This tells us if were showing the form on the dashboard or a post
		global $easygeo_options;
		
		if ($useCurrent==true){
			$loc = $easygeo_options['current_loc'];
			$lat = $easygeo_options['current_lat'];
			$lon = $easygeo_options['current_lon'];
		} else {
			list($loc, $lat, $lon) = EasyGeo::getCoordinates();	
		}
		
		echo "
			<div id='easygeo_map' style='height:300px; width:300px; padding:0px; margin:0px; margin-bottom:-300px;'></div>
			<div style='height:300px; margin-left:315px;'>
			<span style='font-style:italic;'>Enter an address, city, intersection, or anything Google Maps will understand then click the button. Simple.</span>
			<br/><br/>
			<span style='font-weight:bold;'>Location</span><br/><br/>
			<input name='easygeo_loc' id='easygeo_loc' type='text' value='$loc' style='width:100%; margin-right: 10px;' />
			<input type='button' id='easygeo_search_button' name='easygeo_search_button' value='Geocode it!' onclick='easygeoGeocode();' class='button' />
			<span id='easygeo_geocode_status' style='display:none;'>Geocoding...</span>
			<br/><br/>
			<input name='easygeo_update' id='easygeo_update' type='hidden' value='false' />
			<input name='easygeo_lat' id='easygeo_lat' type='hidden' value='$lat' />
			<input name='easygeo_lon' id='easygeo_lon' type='hidden' value='$lon' />
			<span style='color:#aaa;'>LAT&nbsp;=&nbsp;<input name='easygeo_lat_dis' id='easygeo_lat_dis' type='text' disabled='disabled' value='$lat' style='background-color:#eee;' /><br/>
			LON&nbsp;=&nbsp;<input name='easygeo_lon_dis' id='easygeo_lon_dis' type='text' disabled='disabled' value='$lon' style='background-color:#eee;' /></span><br/>
			<input type='button' id='easygeo_clear_button' name='easygeo_clear_button' value='Clear Location' onclick='easygeoClear();' class='button' />
			<br/><br/>";
		if (!$useCurrent && $easygeo_options['track_current']) echo "
			<input type='checkbox' id='easygeo_updatecurrent' name='easygeo_updatecurrent'>&nbsp;&nbsp;&nbsp;<label for='easygeo_updatecurrent'>Set my current location to this location on publish/update</label>
			<br/><br/>";
		echo "
			<span style='color:#aaa; font-style:italic;'>EasyGeo was created by <a href='http://theoephraim.com'>Theo Ephraim</a>. Go say hello if you like it!</span>
			</div>";
	}
	
	function displayOptions() {
		global $easygeo_options;
		echo "
			<div class='wrap'>
				<h2>EasyGeo Configuration</h2>
				<form method='post' action='options.php'>";
				wp_nonce_field("update-options");
		echo "
				<table class='form-table'>
					<tr valign='top'>
						<th scope='row'>Google API Key</th>
						<td><input name='easygeo_options[gmap_api_key]' type='text' value='".$easygeo_options["gmap_api_key"]."' size='50' ";
		if (empty($easygeo_options["gmap_api_key"])) {echo "style='border: 1px solid #d54e21; background-color: #ffebe8;'";}
		echo " /><br />
							You need a unique Google API Key for your website to display Google Maps. If you don't have one yet, <a href='http://code.google.com/apis/maps/signup.html' target='_blank'>sign up</a> to get one!</td>
					</tr>
					<tr valign='top'>
						<th scope='row'>Add Geotags</th>
						<td>".EasyGeo::displayOptions_Checkbox("easygeo_options", array("GEOTAG_FEEDS" => "Add geographical information to feeds"), $easygeo_options)."
							".EasyGeo::displayOptions_Checkbox("easygeo_options", array("GEOTAG_HTML" => "Add geographical information to HTML Header"), $easygeo_options)."</td>
					</tr>
					<tr valign='top'>
						<th scope='row'>Track Current Location</th>
						<td>".EasyGeo::displayOptions_Checkbox("easygeo_options", array("track_current" => "Adds a geotagging widget to the dashboard for tracking a single location (where am I now)"), $easygeo_options)."
					</tr>
				</table>
				<p class='submit'>
					<input type='submit' name='submit' value='Save Changes' class='button-primary' />
					<input type='hidden' name='action' value='update' />
					<input type='hidden' name='page_options' value='easygeo_options' />
				</p>
				</form>
			</div>";
			//echo "<pre>"; print_r($easygeo_options); echo "</pre>";
	}
	
	function displayOptions_Checkbox($name, $options, $selected=null) {
		$output = null;
		foreach ($options as $key => $val) {
			$output .= "<input name='".$name."[".$key."]' type='checkbox' value='true' style='margin-right: 5px;'";
			if ($selected[$key] == "true") {$output .= " checked='checked'";}
			$output .= " />".$val."<br />";
		}
		return $output;
	}
	
	function hookWPHeader() {
		//require jquery! or figure out something else for initializing the map
		global $easygeo_options;
		wp_enqueue_script("jquery");		
		if ($easygeo_options["GEOTAG_HTML"] == "true") {
			list($loc, $lat, $lon) = EasyGeo::getCoordinates();
			if (!is_null($lat) && !is_null($lon)) {
				echo "
					<meta name='geo.position' content='$lat;$lon' />
					<meta name='ICBM' content='$lat, $lon' />";
			}
		}
	}
	
	/* ==================================================================== */
	/* = Dashboard Box                                                    = */
	/* ==================================================================== */
	
	function hookDashboardSetup() {
		global $easygeo_options;
		if ($easygeo_options['track_current']) {
			wp_add_dashboard_widget( 'easygeo_dashbox', __( 'EasyGeo: Current Location' ), array('EasyGeo', 'displayDashboardBox'), array('EasyGeo', 'dashboardCallback') );
		}
	}
	
	/**
	 * Content of Dashboard-Widget
	 */
	function displayDashboardBox() {
		EasyGeo::displayEditPostForm(null, null, true);
	}
	
	function hookAjaxUpdateCurrent() {
		if ( isset($_POST["loc"]) && isset($_POST["lat"]) && isset($_POST["lon"]) ) {
			EasyGeo::putCurrentCoordinates($_POST["loc"], $_POST["lat"], $_POST["lon"]);
		}
	}
	
	/* ==================================================================== */
	/* = Feeds                                                            = */
	/* ==================================================================== */
	
	function hookFeedNamespace() {
		echo "xmlns:georss=\"http://www.georss.org/georss\"";
	}
	
	function hookFeedItem() {
		if (EasyGeo::getCoordinates()) {
			list ($loc, $lat, $lon) = EasyGeo::getCoordinates();
			echo "<georss:point>".$lat." ".$lon."</georss:point>";
		}
	}
	
	
	/* ==================================================================== */
	/* = Auxiliary functions                                              = */
	/* ==================================================================== */
	
	function getCoordinates($post_id=null) {
		global $easygeo_options, $post;
		if (is_null($post_id)) {
			$post_id = $post->ID;
		}
		$loc = get_post_meta($post_id, "easygeo_location", true);
		$lat = get_post_meta($post_id, "easygeo_latitude", true);
		$lon = get_post_meta($post_id, "easygeo_longitude", true);

		if (empty($lat) || empty($lon) || empty($loc)) return null;
		else return array($loc, $lat, $lon);
	}
	function getCurrentCoordinates() {
		global $easygeo_options;
		
		$loc = $easygeo_options['current_loc'];
		$lat = $easygeo_options['current_lat'];
		$lon = $easygeo_options['current_lon'];

		if (empty($lat) || empty($lon) || empty($loc)) return null;
		else return array($loc, $lat, $lon);
	}
	function getCoordinatesString($post_id=null) {
		global $easygeo_options, $post;
		if (is_null($post_id)) {
			$post_id = $post->ID;
		}
		$lat = get_post_meta($post_id, "easygeo_latitude", true);
		$lon = get_post_meta($post_id, "easygeo_longitude", true);
		
		if (empty($lat) || empty($lon)) return null;
		else return $lat.','.$lon;
	}
	function getCurrentCoordinatesString() {
		global $easygeo_options;
		
		$lat = $easygeo_options['current_lat'];
		$lon = $easygeo_options['current_lon'];

		if (empty($lat) || empty($lon)) return null;
		else return $lat.','.$lon;
	}
	function getLocationString($post_id=null) {
		global $easygeo_options, $post;
		if (is_null($post_id)) {
			$post_id = $post->ID;
		}
		return get_post_meta($post_id, "easygeo_location", true);
	}
	function getCurrentLocationString() {
		global $easygeo_options;
		return $easygeo_options['current_loc'];
	}
	
	function putCoordinates($loc=null, $lat=null, $lon=null, $post_id=null) {
		global $easygeo_options, $post;
		if (is_null($post_id)) {
			$post_id = $post->ID;
		}
		if (!empty($lat) && !empty($lon) && !empty($loc)) {
			// Save coordinates
			add_post_meta($post_id, "easygeo_location", $loc, true) or update_post_meta($post_id, "easygeo_location", $loc);
			add_post_meta($post_id, "easygeo_latitude", $lat, true) or update_post_meta($post_id, "easygeo_latitude", $lat);
			add_post_meta($post_id, "easygeo_longitude", $lon, true) or update_post_meta($post_id, "easygeo_longitude", $lon);
		} else {
			// Delete coordinates
			delete_post_meta($post_id, "easygeo_location");
			delete_post_meta($post_id, "easygeo_latitude");
			delete_post_meta($post_id, "easygeo_longitude");
		}
	}
	function putCurrentCoordinates($loc=null, $lat=null, $lon=null ) {
		global $easygeo_options;
		if (!empty($lat) && !empty($lon) && !empty($loc)) {
			// Save coordinates
			$easygeo_options['current_loc'] = $loc;
			$easygeo_options['current_lat'] = $lat;
			$easygeo_options['current_lon'] = $lon;
			update_option("easygeo_options", $easygeo_options);
		}
	}
}
?>